AccountSettingsXMLObj = function(xmlPath)
{
    this.xmlPath = xmlPath;
    this.xmlDoc = new ActiveXObject("Msxml.DOMDocument");
    this.contactsNode = null;
    this.groupsNode = null;
    this.load();
}

AccountSettingsXMLObj.prototype =
{
    load: function()
    {
        this.xmlDoc.load(this.xmlPath);

        // Verify if the file exists first
        this.rootNode = this.xmlDoc.selectSingleNode("//MessengerGadgetAccountSettings");

        if (this.rootNode === null)
        {
            // If the node does not exist it means that there is no file
            this.createNewFile();
            this.rootNode = this.xmlDoc.selectSingleNode("//MessengerGadgetAccountSettings");
        }

        // Verify if contacts node exist
        this.contactsNode = this.xmlDoc.selectSingleNode("//MessengerGadgetAccountSettings/Contacts");

        if (this.contactsNode === null)
        {
            this.addChild(this.rootNode, "Contacts");
            this.contactsNode = this.xmlDoc.selectSingleNode("//MessengerGadgetAccountSettings/Contacts");
        }

        // Verify if groups node exist
        this.groupsNode = this.xmlDoc.selectSingleNode("//MessengerGadgetAccountSettings/Groups");

        if (this.groupsNode === null)
        {
            this.addChild(this.rootNode, "Groups");
            this.groupsNode = this.xmlDoc.selectSingleNode("//MessengerGadgetAccountSettings/Groups");
        }
    },

    createNewFile: function()
    {
        var root = this.xmlDoc.createElement("MessengerGadgetAccountSettings")
        this.xmlDoc.appendChild(root);
    },

    // Add setting methods
    addDisplayName: function(contactId, displayName)
    {
        this.addSettingItem(this.contactsNode, "Contacts", "Contact", contactId, "DisplayName", displayName);
    },

    addGroupCollapsed: function(groupId, groupCollapsed)
    {
        this.addSettingItem(this.groupsNode, "Groups", "Group", groupId, "GroupCollapsed", groupCollapsed);
    },

    // Retreive setting value methods
    getDisplayName: function(contactId)
    {
        return this.getSettingItemValue("Contacts", "Contact", contactId, "DisplayName");
    },

    getGroupCollapsed: function(groupId)
    {
        return this.getSettingItemValue("Groups", "Group", groupId, "GroupCollapsed");
    },

    // Remove setting methods
    removeDisplayName: function(contactId)
    {
        this.removeSettingNode("Contacts", "Contact", contactId, "DisplayName");
    },

    removeGroupCollapsed: function(groupId)
    {
        this.removeSettingNode("Groups", "Group", groupId, "GroupCollapsed");
    },

    // Helper methods
    removeSettingNode: function(settingName, settingItemName, itemId, propertyName)
    {
        var nodeToRemove = this.xmlDoc.selectSingleNode("//MessengerGadgetAccountSettings/" + settingName + "/" + settingItemName + "[@id='" + itemId + "']/" + propertyName);
        var settingNodeToRemove = nodeToRemove.parentNode;
        this.removeNode(nodeToRemove);

        if (!settingNodeToRemove.childNodes.length)
        {
            this.removeNode(settingNodeToRemove);
        }
        this.save();
    },

    addSettingNode: function(settingParentNode, settingItemName, itemId)
    {
        var newContactNode = this.addChild(settingParentNode, settingItemName);
        var idAttr = this.xmlDoc.createAttribute("id");
        idAttr.value = itemId;
        newContactNode.attributes.setNamedItem(idAttr);

        return newContactNode;
    },

    getSettingItemValue: function(settingName, settingItemName, itemId, propertyName)
    {
        this.load();

        var settingNode = this.xmlDoc.selectSingleNode("//MessengerGadgetAccountSettings/" + settingName + "/" + settingItemName + "[@id='" + itemId + "']/" + propertyName);

        if (settingNode == null)
        {
            return null;
        }

        return settingNode.text;
    },

    addSettingItem: function(settingParentNode, settingName, settingItemName, itemId, propertyName, propertyValue)
    {
        var settingNode = this.xmlDoc.selectSingleNode("//MessengerGadgetAccountSettings/" + settingName + "/" + settingItemName + "[@id='" + itemId + "']");
        if (settingNode === null)
        {
            settingNode = this.addSettingNode(settingParentNode, settingItemName, itemId);
        }

        var propertyNode = settingNode.selectSingleNode(propertyName);
        if (propertyNode == null)
        {
            propertyNode = this.addChild(settingNode, propertyName);
        }

        propertyNode.text = propertyValue;

        this.save();
    },

    save: function()
    {
        debug(this.xmlPath);
        try
        {
            this.xmlDoc.save(this.xmlPath);
        }
        catch (ex)
        {
            // Folder does not exist
            debug("this.xmlDoc.save() failed with exception: " + ex);
            debug("creating folder...");
            debug(System.Shell.knownFolderPath("Profile"));
            System.Shell.Folder.newFolder(System.Shell.knownFolderPath("Profile") + "\\appdata\\roaming\\MessengerGadget\\");
            debug("folder created");
            this.contactsXml = new AccountSettingsXMLObj(System.Shell.knownFolderPath("Profile") + "\\appdata\\Roaming\\MessengerGadget\\" + messenger.MySigninName + ".xml");
            debug("trying to open xml file...");
        }
    },

    addChild: function(parentNode, childNodeName)
    {
        var tempNode = this.xmlDoc.createElement(childNodeName);
        debug("parentNode:" + parentNode);
        debug("tempNode:" + tempNode);
        parentNode.appendChild(tempNode);

        return tempNode;
    },

    removeNode: function(nodeToRemove)
    {
        if (nodeToRemove != null)
        {
            nodeToRemove.parentNode.removeChild(nodeToRemove);
        }
    }
}