///////////////////////////////////////////////
//
// javascript library to help building wifimonitor flyout
//
///////////////////////////////////////////////

function trim(stringToTrim) {
    return stringToTrim.replace(/^\s+|\s+$/g, "");
}

function SortableTableBuilder() {
    //variables
    var tableHeaderClass = "tableCellHeader";
    var tableRowClass = "tableCell";
    var tableSpecialRowClass = "tableCellConnected";
    var tableAlternateRowClass = "tableCellAlternate";
    var imgSortAsc = "images/SortAsc.png";
    var imgSortDesc = "images/SortDesc.png";
    var mouseOverColor = "FF8040";
    var mouseLeaveColor = "003445";

    var arrTableNames = new Array();
    var arrTableAtributes = new Array();  //sortcolumn, sortdirection, externalsort flag
    var arrSkinColors = new Array();
    var cellActions = null;
    var locateSSID = null;
    var locateBSSID = null;

    ///function pointers
    this.BuildTable = BuildTable;

    this.SetSortColumn = SetSortColumn;
    this.GetSortColumn = GetSortColumn;

    this.SetSortDirection = SetSortDirection;
    this.GetSortDirection = GetSortDirection;

    this.SetTableHeaderClass = SetTableHeaderClass;
    this.GetTableHeaderClass = GetTableHeaderClass;

    this.SetTableRowClass = SetTableRowClass;
    this.GetTableRowClass = GetTableRowClass;
    this.SetTableAlternateRowClass = SetTableAlternateRowClass;
    this.GetTableAlternateRowClass = GetTableAlternateRowClass;
    this.SetSortAttributes = SetSortAttributes;
    this.GetSortAttributes = GetSortAttributes;

    this.GetExternalSort = GetExternalSort;
    this.SetExternalSort = SetExternalSort;
    this.GetSkinColors = GetSkinColors;
    this.SetSkinColors = SetSkinColors;
    this.SetCellActions = SetCellActions;
    this.SetLocateParams = SetLocateParams;

    function SetLocateParams(params) {
        if (null != params) {
            var arrParams = params.split('|');
            locateSSID = arrParams[0];
            locateBSSID = arrParams[1];
            //System.Debug.outputString("SetLocateParams( ssid ='" + locateSSID+"' bssid='"+ locateBSSID+"')")
        } else {
            locateSSID = null;
            locateBSSID = null;
            //System.Debug.outputString("SetLocateParams( null )");
        }
    }

    function SetCellActions(ca) {
        cellActions = ca;
    }

    function GetSkinColors() {
        return arrSkinColors;
    }

    function SetSkinColors(ArraySkinColors) {
        arrSkinColors = ArraySkinColors;
        mouseOverColor = ArraySkinColors[15];
        mouseLeaveColor = ArraySkinColors[2];
    }

    function GetTableAlternateRowClass() {
        return tableAlternateRowClass;
    }

    function SetTableAlternateRowClass(aClassName) {
        tableAlternateRowClass = aClassName;
    }

    function GetTableRowClass() {
        return tableRowClass;
    }

    function SetTableRowClass(aClassName) {
        tableRowClass = aClassName;
    }

    function GetTableHeaderClass() {
        return tableHeaderClass;
    }

    function SetTableHeaderClass(aClassName) {
        tableHeaderClass = aClassName;
    }

    function SetSortDirection(tablename, dir) {
        var sortAttributes = GetSortAttributes(tablename)
        sortAttributes[1] = dir;
    }

    function GetSortDirection(tablename) {
        var sortAttributes = GetSortAttributes(tablename)
        return sortAttributes[1];
    }

    function SetSortColumn(tablename, colname) {
        var sortAttributes = GetSortAttributes(tablename);
        sortAttributes[0] = colname;
    }

    function GetSortColumn(tablename) {
        var sortAttributes = GetSortAttributes(tablename)
        return sortAttributes[0];

    }
    function GetExternalSort(tablename) {
        var sortAttributes = GetSortAttributes(tablename)
        return sortAttributes[2];

    }

    function SetExternalSort(tablename, isexternal) {
        var sortAttributes = GetSortAttributes(tablename)
        sortAttributes[2] = isexternal;
    }

    function SetSortAttributes(tablename, sortcolumn, sortdirection) {
        for (var i = 0; i < arrTableNames.length; ++i) {
            if (arrTableNames[i] == tablename) {
                arrTableAtributes[i] = new Array(sortcolumn, sortdirection, false);
                return;
            }
        }
        var idx = arrTableNames.length;
        arrTableNames[idx] = tablename;
        arrTableAtributes[idx] = new Array(sortcolumn, sortdirection, false);
    }

    function GetSortAttributes(tablename) {
        for (var i = 0; i < arrTableNames.length; ++i) {
            if (arrTableNames[i] == tablename) {
                return arrTableAtributes[i];
            }

        }
        return null;
    }

    function makeColHeaderCell(tablename, collname, colWidth, sc, dir) {

        var width = "";
        if (colWidth != null) {
            width = "width:" + colWidth + "px; ";
        }
        var dname = collname;
        var sortType = "<img src='" + imgSortAsc + "' alt='Click to sort descending' />";
        if (collname == sc) {
            if (!dir) {
                sortType = "<img src='" + imgSortDesc + "' alt='Click to sort ascending'/>"; ;
            }
            dname += sortType;
        }
        var thetable = '"' + tablename + '"';
        var thecolumn = '"' + collname + '"';

        var clickaction = "onclick='OnSortColumn(" + thetable + "," + thecolumn + ", " + !dir + ")' ";
        var mouseenteraction = "onmouseover='this.style.backgroundColor=\"#" + mouseOverColor + "\";' ";
        var mouseleaveaction = "onmouseout='this.style.backgroundColor=\"#" + mouseLeaveColor + "\";' ";
        var splitidx = dname.indexOf("(");
        if (-1 != splitidx) {
            var cols = dname.split('(');
            dname = trim(cols[0]) + "&nbsp;<br/>(" + cols[1];
        }

        var cell = "<td id='headerCell" + collname + "' " + clickaction + mouseenteraction + mouseleaveaction + " style='" + width + "background-color:#" + getHeaderBkColor() + "; color:#" + getHeaderColor() + ";'>" + dname + " </td>";
        //System.Debug.outputString("makeColHeaderCell : "+cell);
        return cell;
    }

    function getHeaderColor() {
        return arrSkinColors[3];
    }

    function getHeaderBkColor() {
        return arrSkinColors[2];
    }

    function getRowBkColor(rowidx, idxSpecialRow) {
        if (rowidx == idxSpecialRow) {
            return arrSkinColors[7];
        }

        if (0 == rowidx % 2) {
            return ""; //alternate 
        }
        return arrSkinColors[6];
    }

    function getRowForeColor(rowidx, idxSpecialRow) {
        if (rowidx == idxSpecialRow) {
            return arrSkinColors[8];
        }

        if (0 == rowidx % 2) {
            return arrSkinColors[16];
        }
        return arrSkinColors[5];
    }

    function getRowClass(rowidx, idxSpecialRow) {
        var rowClass = "";
        if (0 == rowidx % 2) {
            rowClass = tableRowClass;
        } else {
            rowClass = tableAlternateRowClass;
        }
        if (rowidx == idxSpecialRow) {
            rowClass = tableSpecialRowClass;
        }
        return rowClass;
    }

    function isStringGreater(a, b, sortdir) {


        var la = a.toLowerCase();
        var lb = b.toLowerCase();
        /*
        while(la.length < lb.length){
        la += ' ';
        }
        while(lb.length < la.length)
        {
        lb += ' ';
        }
        */
        //System.Debug.outputString(" compare '"+la+"' vs '"+lb+"' dir="+sortdir);
        if (sortdir) {
            if (la > lb) {
                return true;
            } else {
                return false;
            }
        } else {
            if (la < lb) {
                return true;
            } else {
                return false;
            }
        }
    }

    function isRowGreater(a, b, idx, sortdir) {
        var cellValuesa = a[idx].split('|');
        var cellValuesb = b[idx].split('|');
        var vala = cellValuesa[0].toLowerCase();
        var valb = cellValuesb[0].toLowerCase();

        if (0 == idx) {
            var res = isStringGreater(vala, valb, sortdir);
            return res;
        }
        else {

            if (sortdir) {
                if (vala > valb) {
                    return true;
                } else {
                    return false;
                }
            } else {
                if (vala < valb) {
                    return true;
                } else {
                    return false;
                }
            }
        }
    }

    function SortRows(arrColumns, arrRows, sortcolumn, sortdir) {
        //determine column index;
        var idx = arrColumns.length;
        for (var i = 0; i < arrColumns.length; ++i) {
            if (arrColumns[i] == sortcolumn) {
                idx = i;
            }
        }
        if (idx == arrColumns.length) {
            //column not found;
            return;
        }
        //simple bubble sort
        var rx, ry, holder;
        var flag;
        for (rx = arrRows.length - 1; rx > 0; rx--) {
            flag = true;
            for (ry = 0; rx > ry; ry++) {
                if (isRowGreater(arrRows[ry], arrRows[ry + 1], idx, sortdir)) {
                    flag = false;
                    holder = arrRows[ry];
                    arrRows[ry] = arrRows[ry + 1];
                    arrRows[ry + 1] = holder;

                }
            }
            if (flag) {
                break;
            }
        }
    }

    function GetAction(row, cel) {

        if (null == cellActions) return "";

        if (cellActions.length < row) return "";
        var ra = cellActions[row];

        if (null == ra) return "";

        if (ra.length <= cel) return "";

        if (ra[cel] == null) return "";

        return ra[cel];

    }

    function isLocateMode() {
        return null != locateSSID;
    }

    function isLocateRow(row) {
        if (null == row) return false;
        var ssidcell = row[0];
        var bssidcell = row[6];
        if (row.length < 7) return false;

        var ret = (-1 != bssidcell.indexOf(locateBSSID));

        //if(ret){

        //System.Debug.outputString("ssid: " + ssidcell +"--- \r\nlocateBSSID: '"+locateBSSID+"' bssidcell='"+bssidcell+"'\r\n ret:"+ret);
        //}
        return ret;
    }

    /// arrColumns array of string containng column Names
    /// arrRows array of array of cell contents
    function BuildTable(tablename, arrColumns, arrRows, idxSpecialRow) {
        var output = "";
        output += "<table id='" + tablename + "'><tr class='" + GetTableHeaderClass() + "'>";
        //System.Debug.outputString("BuildTable got "+arrColumns.length+" columns");
        //System.Debug.outputString("BuildTable got "+arrRows.length+" rows");
        //System.Debug.outputString("BuildTable got "+idxSpecialRow+" as idxSpecialRow");
        var tableSortAttributes = GetSortAttributes(tablename);
        if (null == tableSortAttributes) {
            //make defaults
            SetSortAttributes(tablename, arrColumns[0], false);
            tableSortAttributes = GetSortAttributes(tablename);
        }
        for (var c = 0; c < arrColumns.length; c++) {
            var collnameData = arrColumns[c].split('|');
            var colWidth = null;
            if (collnameData.length > 1) {
                colWidth = collnameData[1];
            }
            output += makeColHeaderCell(tablename, collnameData[0], colWidth, GetSortColumn(tablename), GetSortDirection(tablename));
        }
        output += "</tr>";

        if (false == tableSortAttributes[2]) {  //not a external sort
            //sort the rows here
            var column = tableSortAttributes[0];
            var dir = tableSortAttributes[1];
            //System.Debug.outputString("Table "+ tablename +"is not external sort");
            //System.Debug.outputString("JS sorting table '"+tablename+"' by column '"+column+"' direction "+dir);
            SortRows(arrColumns, arrRows, column, dir);
        }
        /*
        else{
        System.Debug.outputString("Table "+ tablename +"is external sort");
        }*/

        for (var r = 0; r < arrRows.length; r++) {
            var row = arrRows[r];
            if (null == row) continue;
            //System.Debug.outputString("BuildTable got row "+r+" with "+ row.length +" cels");
            var rowClass = getRowClass(r, idxSpecialRow);

            if (!isLocateMode()) {
                output += "<tr class='" + rowClass + "' style='background-color:#" + getRowBkColor(r, idxSpecialRow) + "; color:#" + getRowForeColor(r, idxSpecialRow) + ";'>";
            }
            else {

                if (!isLocateRow(row)) {
                    output += "<tr class='" + rowClass + "' style ='background-color:#" + getRowBkColor(r, idxSpecialRow) + "; color:#" + getRowForeColor(r, idxSpecialRow) + ";'>";
                } else {
                    output += "<tr class='" + rowClass + "' style ='background-color: yellow; color: black;'>";
                }
            }

            if (null == row || 0 == row.length) continue;
            for (var rc = 0; rc < row.length; ++rc) {
                var cell = row[rc];
                //System.Debug.outputString("BuildTable got cell ["+r +","+rc+"]="+cell);

                var cellValues = cell.split('|');

                var actions = GetAction(r, rc);
                if (cellValues[1] == "none") {
                    output += "<td " + actions + ">";
                }
                else {
                    output += "<td title='" + cellValues[2] + "' " + actions + ">";
                }

                if (cellValues.length < 3) {
                    if (cellValues[1] != "none") {
                        output += "<img src='" + cellValues[1] + "' align='middle'/>&nbsp;";
                    }
                } else {
                    if (cellValues[1] != "none") {
                        //if(cellValues[1] == "u")
                        output += "<img src='" + cellValues[1] + "' alt='" + cellValues[2] + "' align='middle'/>&nbsp;";
                    }

                }

                if (null != actions && actions.length != "") {
                    output += "<u>";
                }

                output += cellValues[0];
                if (null != actions && actions.length != "") {
                    output += "</u>";
                }
                output += "</td>";
            }
            output += "</tr>";
        }
        output += "</table>";
        //System.Debug.outputString(output);
        return output;
    }
}


function FlyoutBuilder() {
    //variables
    var wrapi;
    var columnNames;
    var theDocument;
    var sortableTableBuilder = new SortableTableBuilder();
    var arrSkinColors = null;
    var tabBuilder;

    var openedAdapterIndex = null;

    this.GetWrapi = GetWrapi;
    this.SetWrapi = SetWrapi;
    this.SetDocument = SetDocument;
    this.GetDocument = GetDocument;
    this.BuildFlyout = BuildFlyout;
    this.SetTabBuilder = SetTabBuilder;
    this.GetTabBuilder = GetTabBuilder;
    this.GetTableBuilder = GetTableBuilder;
    this.SetOpenAdapterIndex = SetOpenAdapterIndex;
    this.SetSkinColors = SetSkinColors;
    this.GetSkinColors = GetSkinColors;


    function GetSkinColors() {
        return arrSkinColors;
    }

    function SetSkinColors(ArraySkinColors) {
        arrSkinColors = ArraySkinColors;
        if (null != tabBuilder) {
            tabBuilder.SetSkinColors(ArraySkinColors);
            sortableTableBuilder.SetSkinColors(ArraySkinColors);
        }
    }



    function SetOpenAdapterIndex(index) {
        openedAdapterIndex = index;
    }

    function GetTabBuilder() {
        return tabBuilder;
    }

    function SetTabBuilder(aTabBuilder) {
        tabBuilder = aTabBuilder;
        if (null != GetSkinColors()) {
            tabBuilder.SetSkinColors(GetSkinColors());
        }
    }


    function GetTableBuilder() {
        return sortableTableBuilder;
    }

    function GetWrapi() {
        return wrapi;
    }

    function SetWrapi(value) {
        wrapi = value;
    }

    function SetDocument(aDocument) {
        theDocument = aDocument;
    }
    function GetDocument() {
        return theDocument;
    }

    function getChannelName(net) {

        if (!net.Has11nInformation) {
            var frequency = net.Frequency / 1000.0;
            var channelid = GetWrapi().FrequencyToChannel(frequency);

            if (-1 == channelid) { return "Unknown"; }

            return "" + channelid;
        } else {
            if (net.SecondaryChannel > 0) {
                return "" + net.PrimaryChannel + ", " + net.SecondaryChannel;
            }
            return "" + net.PrimaryChannel;
        }

    }

    function getFrequency(net) {
        if (!net.Has11nInformation) {
            return net.Frequency / 1000;
        }
        else {
            return net.FrequencyS;
        }
    }

    function PrepareForWrap(content_string) {
        var splitcount = 10;
        var result = "";
        if (content_string.length <= 12) return content_string;
        //result += '<br />';
        for (var i = 0; i < content_string.length; ++i) {

            var c = content_string.charAt(i);
            if (0 == i % splitcount) {
                result += '<br />';
                splitcount = 15;
            }
            result += c;
        }
        return result;
    }


    function makeAvaillableNetworksTable(tablename, adapterItem, networks, ai) {

        var attrs = sortableTableBuilder.GetSortAttributes(tablename);
        var first_connected = false;
        if (null == attrs) {
            //majke default rssi
            sortableTableBuilder.SetSortAttributes(tablename, "RSSI (dBm)", true);
            attrs = sortableTableBuilder.GetSortAttributes(tablename);
            first_connected = true;
        }
        sortableTableBuilder.SetExternalSort(tablename, true);
        var arrColumnNames = new Array(
            "SSID",
            "Default Authentication",
            "Default Encryption",
            "RSSI (dBm)|55",
            "Channel",
            "Frequency (MHz)",
            "BSSID (MAC Address)",
            "Network Mode",
            "Network Type"
        );

        var sortcolumn = sortableTableBuilder.GetSortColumn(tablename);
        var sortdir = sortableTableBuilder.GetSortDirection(tablename);
        //System.Debug.outputString("availlable net sorted by "+ sortcolumn);

        networks.SortBy(sortcolumn, sortdir);

        var arrRows = new Array();
        var nc = networks.NetworkCount;
        var idxConnected = 10000; //default to large value for not connected case
        var connectedNetwork = null;
        var rowConnected = null;
        var actions = new Array();
        for (var n = 0; n < nc; ++n) {
            try {
                var net = networks.GetAt(n);
                if (null == net | undefined == net) continue;

                var privacyImage = "images/decrypted.png|Click to locate";
                if (net.Privacy != 0) {
                    privacyImage = "images/encrypted.png|Click to locate";
                }

                var secpair = net.DefaultSecutityAlgorithms.split(',');
                var auth = secpair[0];
                var chiper = secpair[1];
                var SSIDValue = "";
                SSIDValue = net.SSID;
                SSIDValue.replace(" ", "");

                //System.Debug.outputString("SSIDValue='"+SSIDValue+"' length: "+SSIDValue.length);
                if (0 == SSIDValue.length) {
                    SSIDValue = "&lt;Non-broadcasted&gt;";

                } else {
                    SSIDValue = PrepareForWrap(SSIDValue);
                }

                //System.Debug.outputString("final SSIDValue='"+SSIDValue+"' length: "+SSIDValue.length);

                var rssi_image = GetWrapi().GetRssiImage(System.Gadget.path + "\\images\\", "rssi.png", net.RSSI);
                //System.Debug.outputString("rssi_image='"+rssi_image+"' from :'"+ System.Gadget.path+"\\images\\'");

                var rowCels = new Array(
                    SSIDValue + "|" + privacyImage,
                    auth + "|none",
                    chiper + "|none",
                    net.RSSI + "|images/" + rssi_image + "|",
                    getChannelName(net) + "|none",
                    getFrequency(net) + "|none",
                    net.DecodedMacAddress + "|images/u.bmp|" + net.MacAddress,
                    net.NetworkMode + "|none",
		            net.NetworkType + "|none"
                );

                var enterlocateactions = "onclick='DoEnterLocateMode(\"" + net.SSID + "\", \"" + net.MacAddress + "\", " + ai + ");' onmouseover ='this.style.cursor = \"hand\";' onmouseout ='this.style.cursor = \"hand\";' ";
                if (ai == System.Gadget.Settings.read("adapterIndex")) {
                    actions[n] = new Array(
                                enterlocateactions,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null
                    );
                } else {
                    actions[n] = new Array(
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null,
                                null
                    );
                }

                var include = true;

                if (adapterItem.IsConnectedNetwork(net)) {
                    idxConnected = n;
                    if (first_connected) {
                        connectedNetwork = net;
                        rowConnected = rowCels;
                        include = false;
                    }
                }

                arrRows[n] = rowCels;
            } catch (e) {
                //System.Debug.outputString("exception:"+e);
            }

        }

        var ret;

        if (adapterItem.IsLocateMode()) {
            var locateParams = adapterItem.GetLocateParams();
            sortableTableBuilder.SetLocateParams(locateParams);
        } else {
            sortableTableBuilder.SetLocateParams(null);
        }

        //System.Debug.outputString("makeAvaillableNetworksTable");

        if (first_connected) {
            var finalActions = new Array();
            finalActions[0] = actions[idxConnected];

            var finalArray = new Array();

            finalArray[0] = rowConnected;
            var destidx = 1;

            for (var r = 0; r < arrRows.length; ++r) {
                if (r == idxConnected) {
                    continue;
                }
                finalArray[destidx] = arrRows[r];
                finalActions[destidx] = actions[r];
                destidx++;
            }
            sortableTableBuilder.SetCellActions(finalActions);
            ret = sortableTableBuilder.BuildTable(tablename, arrColumnNames, finalArray, 0);

        } else {
            sortableTableBuilder.SetCellActions(actions);
            ret = sortableTableBuilder.BuildTable(tablename, arrColumnNames, arrRows, idxConnected);
        }
        sortableTableBuilder.SetCellActions(null);
        //System.Debug.outputString(ret);
        return ret;


    }

    var termTable = new Array(
        new Array("802.11", "A group of wireless networking standards defined by the Institute of Electrical and Electronics Engineers (IEEE) and commonly referred to as Wi-Fi or WLAN."),
        new Array("802.11a", "A Wi-Fi network standard that describes radio transmissions in the 5.0-5.8GHz frequency range and with data rates of up to 54Mbps."),
        new Array("802.11b", "A Wi-Fi network standard that describes radio transmissions in the 2.4GHz frequency range and with data rates of up to 11Mbps."),
        new Array("802.11g", "A Wi-Fi network standard that describes radio transmissions in the 2.4GHz frequency range and with data rates of up to 54Mbps."),
        new Array("802.11i", "An  IEEE standard that specifies AES or TKIP encryption and 802.1X authentication for securing Wi-Fi networks. It supersedes the previous WEP specification from the original 802.11 standard that was found to be easily compromised."),
        new Array("802.11n", "The latest IEEE Wi-Fi network standard, ratified in September 2009, that describes radio transmissions in both the 2.4GHz and 5GHz frequency ranges and with data rates of up to 600Mbps."),
        new Array("802.1x", "An IEEE standard for port-based network access control, providing for the authentication of users attempting to access a network. It is specified by the IEEE 802.11i standard and the Wi-Fi Alliance WPA and WPA2 certifications for implementing Wi-Fi security. It is typically operated in conjunction with a RADIUS server."),
        new Array("Access Point", "A Wi-Fi device (typically with 1 or 2 radios) that connects wireless devices/users to another (typically wired) network. Commonly abbreviated as AP."),
        new Array("Ad Hoc", "A Wi-Fi network connection method that does not require an access point (base station). Using this mode, Wi-Fi devices such as laptops or gaming stations can connect directly to each other."),
        new Array("AES (Advanced Encryption Standard)", "The preferred encryption algorithm for use in wireless LANs today. It provides government-grade encryption and can be used with both WPA and WPA2 Wi-Fi security."),
        new Array("BSSID (Basic Service Set Identifier)", "The MAC address of the Access Point."),
        new Array("CCMP (Counter Mode with Cipher Block Chaining with Message Authentication Code)", "An encryption protocol defined by IEEE 802.11i. It is used in conjunction with the AES encryption algorithm and is part of both WPA and WPA2 Wi-Fi security."),
        new Array("Channel", "A frequency band, identified by a unique number, used for Wi-Fi communication. Each channel supports independent communication from any other channel. Wi-Fi channels are 20MHz wide in 802.11a/b/g networks, and can be either 20MHz or 40MHz in 802.11n networks."),
        new Array("dBm", "A logarithmic unit of measure for milliwatts of power, used in Wi-Fi to measure the strength of a signal. Several examples of the conversion from dBm to milliwatts: 0 dBm = 1 milliwatt; 10dBm = 10 milliwatts; 20dBm = 100 milliwatts; -10dBm = 0.1 milliwatts."),
        new Array("MAC Address (Media Access Control Address)", "A quasi-unique address value used to identify network adapters that follow different communication standards. In Wi-Fi (as well as Ethernet and other standards), MAC addresses are six bytes (48 bits) in length."),
        new Array("PSK (Pre-shared Key)", "An encryption key shared between and common to the access point and client. It is used in WPA and WPA2 security."),
        new Array("RADIUS (Remote Authentication Dial In User Service)", "An AAA (Authentication, Authorization and Accounting) protocol for controlling user access to a wired or wireless network."),
        new Array("Roaming", "The ability for a mobile wireless station to transparently change its connection between access points as it moves throughout a wireless network."),
        new Array("RSSI (Receive Signal Strength Indication)", "The strength of the Wi-Fi signal as measured by the receiver of the signal. The larger the signal strength, the better the connection. RSSI is usually expressed in dBm or as a numerical percentage. The translation of dBm to percentage is: -100dBm = 0% and -50dBm = 100%, with each dBm accounting for 2% in between. A good Wi-Fi signal is typically considered to be -70dBm or greater (keep in mind negative numbers, so -60dBm is greater than -70dBm for example)."),
        new Array("Signal Strength", "See RSSI."),
        new Array("SSID (Service Set Identifier)", "A unique name that identifies a wireless LAN and that differentiates it from others. All access points and clients attempting to connect to a specific WLAN must use the same SSID."),
        new Array("TKIP (Temporal Key Exchange Protocol)", "An encryption protocol defined by 802.11i as an enhancement to WEP. Both WPA and WPA2 Wi-Fi security allow for the use of TKIP encryption."),
        new Array("WEP (Wired Equivalency Protocol)", "The original encryption protocol defined for 802.11 wireless LANs by the IEEE. WEP encryption is easily cracked and is not recommended for implementing Wi-Fi security today."),
        new Array("WEP-40", "WEP with 40 bit master encryption key."),
        new Array("WEP-104", "WEP with 104 bit master encryption key."),
        new Array("WPA (Wireless Protected Access)", "The original Wi-Fi Alliance certification of 802.11i security for wireless LANs. It provides good Wi-Fi security but was introduced an interim option before WPA2 was released."),
        new Array("WPA2 (Wireless Protected Access 2)", "The second generation Wi-Fi Alliance certification of 802.11i security for wireless LANs. Use of WPA2 is considered best practice for implementing Wi-Fi security today."),
        new Array("Wi-Fi", "A term developed by the Wi-Fi Alliance to describe wireless local area network (WLAN) products that are based on the IEEE 802.11 standards."),
        new Array("Wi-Fi Alliance", "Industry organization that certifies 802.11a/b/g/n products for interoperability.")
    );

    function makeTermRow(term) {
        //System.Debug.outputString("Term Name='"+term[0]+"'");
        var rowCells = new Array(
                term[0] + "|none",
                term[1] + "|none"
                );
        return rowCells;
    }

    function makeTerminologyTable(tablename) {
        var attrs = sortableTableBuilder.GetSortAttributes(tablename);

        if (null == attrs) {
            //majke default rssi
            sortableTableBuilder.SetSortAttributes(tablename, "Term Name", true);
        }

        var arrColumnNames = new Array("Term Name", "Definition");

        var arrRows = new Array();
        for (var i = 0; i < termTable.length; ++i) {
            arrRows[i] = makeTermRow(termTable[i]);
        }
        return sortableTableBuilder.BuildTable(tablename, arrColumnNames, arrRows, 10000);
    }

    var MACStatCounterNames = new Array(
          new Array("TransmittedFrameCount", "Transmitted Frame Count", "Number of successfully transmitted packets"),
          new Array("ReceivedFrameCount", "Received Frame Count", "Number of successfully received packets.")
    );

    var MACStstCounterNames1 = new Array(
          new Array("DecryptSuccessCount", "Decrypt Success Count", "Number of encrypted packets that the NIC has successfully decrypted."),
          new Array("DecryptFailureCount", "Decrypt Failure Count", "Number of encrypted packets that the NIC has failed to decrypt."),
          new Array("CCMPReplays", "CCMP Replays", "Number of received unicast fragments discarded by the replay mechanism."),
          new Array("CCMPDecryptErrors", "CCMP Decrypt Errors", "Number of received fragments discarded by the CCMP decryption algorithm."),
          new Array("TKIPLocalMICFailures", "TKIP LocalMIC Failures", "Number of Message Integrity Code (MIC) failures encountered while checking the integrity of packets received."),
          new Array("TKIPReplays", "TKIP Replays", "Number of TKIP replay errors detected."),
          new Array("TKIPICVErrorCount", "TKIP ICV Error Count", "Number of TKIP protected packets that the NIC failed to decrypt."),
          new Array("WEPExcludedCount", "WEP Excluded Count", "Number of frames discarded due to a protected  status in the frame control field."),
          new Array("WEPUndecryptableCount", "WEP Undecryptable Count", "Number of WEP protected packets received for which a decryption key was not available on the NIC."),
          new Array("WEPICVErrorCount", "WEP ICV Error Count", "Number of WEP protected packets the NIC failed to decrypt.")
    );

    function makeStatRow(stats, arrStatVarName) {
        var statVar = arrStatVarName[0];
        var statName = arrStatVarName[1];
        var description = arrStatVarName[2];
        var rowCells = new Array(
                statName + "|none",  //Name
                eval("stats.Get" + statVar + "()") + "|none", //Value
                eval("stats.Get" + statVar + "Rate()") + "|none", //Rate
                description + "|none" //description
                );
        return rowCells;
    }

    function makeStatisticTableMulticast(tablename, adapterItem) {
        var stats = adapterItem.GetStatistics();

        macstats = stats.GetMacMcastCounters();
        var arrColumnNames = new Array("Name", "Value", "Rate (/s)", "Description");

        var arrRows = new Array();
        for (var i = 0; i < MACStatCounterNames.length; ++i) {
            arrRows[i] = makeStatRow(macstats, MACStatCounterNames[i]);
        }
        var table1 = sortableTableBuilder.BuildTable(tablename, arrColumnNames, arrRows, 10000);

        arrRows = new Array();
        for (var i = 0; i < MACStstCounterNames1.length; ++i) {
            arrRows[i] = makeStatRow(macstats, MACStstCounterNames1[i]);
        }
        var table2 = sortableTableBuilder.BuildTable(tablename, arrColumnNames, arrRows, 10000);

        return table1 + "</br>" + table2;
    }


    function makeStatisticTableUnicast(tablename, adapterItem) {
        var stats = adapterItem.GetStatistics();

        macstats = stats.GetMacUcastCounters();
        var arrColumnNames = new Array("Name", "Value", "Rate (/s)", "Description");

        var arrRows = new Array();
        for (var i = 0; i < MACStatCounterNames.length; ++i) {
            arrRows[i] = makeStatRow(macstats, MACStatCounterNames[i]);
        }
        var table1 = sortableTableBuilder.BuildTable(tablename, arrColumnNames, arrRows, 10000);

        arrRows = new Array();
        for (var i = 0; i < MACStstCounterNames1.length; ++i) {
            arrRows[i] = makeStatRow(macstats, MACStstCounterNames1[i]);
        }
        var table2 = sortableTableBuilder.BuildTable(tablename, arrColumnNames, arrRows, 10000);

        return table1 + "</br>" + table2;
    }

    var PhysycalStatNames = new Array(
             new Array("TransmittedFrameCount", "Transmitted Frame Count", "Number of packets successfully transmitted."),
             new Array("ReceivedFrameCount", "Received Frame Count", "Number of packets successfully received.")
    );

    var PhysycalStatNames1 = new Array(

             new Array("MulticastTransmittedFrameCount", "Multicast Transmitted Frame Count", "Number of successfully transmitted packets to a multicast destination MAC address."),
             new Array("FailedCount", "Failed Count", "Number of packet transmission failures due to the number of transmit attempts exceeding the retry limit."),
             new Array("RetryCount", "Retry Count", "Number of packets successfully transmitted after one or more retransmissions."),
             new Array("MultipleRetryCount", "Multiple Retry Count", "Number of packet s successfully transmitted after more than one retransmission."),
             new Array("MaxTXLifetimeExceededCount", "Max TX Lifetime Exceeded Count", "Number of fragmented packets that failed to send due to timeout."),
             new Array("TransmittedFragmentCount", "Transmitted Fragment Count", "Number of packets with an individual address in the address 1 field and packets that have a multicast address with types Data or Management."),
             new Array("RTSSuccessCount", "RTS Success Count", "Number of times a Clear-to-Send (CTS) has been received in response to a Request-to-Send (RTS)."),
             new Array("RTSFailureCount", "RTS Failure Count", "Number of times a Clear-to-Send (CTS) has not been received in response to a Request-to-Send (RTS)."),
             new Array("ACKFailureCount", "ACK Failure Count", "Number of times an expected ACK has not been received."),
             new Array("PromiscuousReceivedFrameCount", "Promiscuous Received Frame Count", "Number of packets successfully received only because promicscuous mode is enabled."),
             new Array("MulticastReceivedFrameCount", "Multicast Received Frame Count", "Number of multicast received frames"),
             new Array("MaxRXLifetimeExceededCount", "Max RX Lifetime Exceeded Count", "Number of fragmented packets dropped due to timeout."),
             new Array("FrameDuplicateCount", "Frame Duplicate Count", "Number of frames received that the Sequence Control field indicates as a duplicate."),
             new Array("ReceivedFragmentCount", "Received Fragment Count", "Number of successfully received Data or Management packets."),
             new Array("PromiscuousReceivedFragmentCount", "Promiscuous Received Fragment Count", "Number of fragments successfully received only because promiscuous mode is enabled."),
             new Array("FCSErrorCount", "FCS Error Count", "Number of times an FCS error has been detected in a received packet.")
    );


    function makeStatisticTablePhysycal(tablenameprefix, adapterItem) {
        var output = "";

        var stats = adapterItem.GetStatistics();

        var arrColumnNames = new Array("Name", "Value", "Rate", "Description");

        var countPhy = stats.NumberOfPhy;
        if (0 == countPhy) {
            output += "</br></br> NA </br>";
            return output;
        }
        //output += "</br></br>Total Physical Tables "+ countPhy +"</br>";

        for (var p = 0; p < countPhy; ++p) {
            var phyStats = stats.GetPhyCounters(p);
            output += "</br></br>Antenna " + p + "</br>";

            var arrRows = new Array();
            for (var i = 0; i < PhysycalStatNames.length; ++i) {
                arrRows[i] = makeStatRow(phyStats, PhysycalStatNames[i]);
            }
            output += sortableTableBuilder.BuildTable(tablenameprefix + i, arrColumnNames, arrRows, 10000);
            output += "</br>";

            arrRows = new Array();
            for (var i = 0; i < PhysycalStatNames1.length; ++i) {
                arrRows[i] = makeStatRow(phyStats, PhysycalStatNames1[i]);
            }
            output += sortableTableBuilder.BuildTable(tablenameprefix + i, arrColumnNames, arrRows, 10000);
            output += "</br>";

        }

        return output;
    }

    function getSupportedRates(net) {
        var output = "";
        var src = net.GetSupportedRatesCount();
        for (var s = 0; s < src; s++) {
            var rate = net.GetSuportedRate(s);
            if (0 == rate) continue;
            output += rate * 0.5;
            if (s < src - 1) {
                output += ",&nbsp;";
            }
        }
        return output;
    }

    function getDataRateInMbps(rate) {
        var rate_in_mbps = (rate & 0x7FFF) * 0.5
        return rate_in_mbps;
    }


    function BuildSingleAdapterOutput(i, adapterItem) {
        if (null == adapterItem) return "";
        if (adapterItem.IsDisabled()) {
            var output = "<table>";
            output += "<tr><td></td><td></td><td></td><td><center> <input type='button' onclick='EnableAdapter(" + i + ");' value='Enable Adapter' title='Click to enable adapter'></input>  </center></td></tr>";
            output += "</table>";
            return output;
        }
        if (adapterItem.SSID == "Not Ready") return "";

        var output = "";
        var adapterHeaderClass = "adapterHeaderOpen";
        var adapterHolderClass = "adapterHolderOpen";
        var adapterBodyClass = "adapterBodyClosed";
        if (null != openedAdapterIndex) {
            if (i == openedAdapterIndex) {
                adapterBodyClass = "adapterBodyOpen";
            }
        }

        var count = GetWrapi().GetNumWireless;
        //System.Debug.outputString("index "+ i+" ->adapterBodyClass = '"+adapterBodyClass+"'");
        if (count == 1) {
            output += "<div id='adapterHolder' class='" + adapterHolderClass + "'>";
            output += "<div id='adapterHeader' class='" + adapterHeaderClass + "'  style='color:#" + GetSkinColors()[12] + ";background-color:#" + GetSkinColors()[13] + ";' onclick='ShowHide(" + i + ");' title='Click to toggle collapsed state'>";
            output += "" + (i + 1) + ". " + adapterItem.Description + "&nbsp; &nbsp;</div>";
        } else {
            adapterBodyClass = "adapterBodyOpen";
        }

        output += "<div id='adapterBody" + i + "' class='" + adapterBodyClass + "'>";
        if (count != 1) {
            output += "<div style='font-size: x-small;color:#" + GetSkinColors()[12] + ";background-color:#" + GetSkinColors()[13] + ";'><center><b>" + adapterItem.Description + "</b></center></div>";
        }
        var disableButton = "";
        if (adapterItem.SSID != "Disconnected") {
            disableButton = "<input type='button' onclick='DisableAdapter(" + i + ");' value='Disable Adapter' title='Click to disable adapter'/>"
        }

        output += "<table style='font-size: x-small;' width='100%'><tr><td>";
        output += "<b>Connected to:&nbsp;</b></td><td>" + adapterItem.SSID + "<br /></td>";
        output += "<td><b>&nbsp;BSSID:&nbsp;</b></td><td title='" + adapterItem.BSSID + "'> " + adapterItem.DecodedBSSID + "&nbsp;<br /></td><td rowspan='5' valign='top'>" + disableButton + "</td></tr>";
        output += "<tr><td><b>Association Mode:&nbsp;</b></td><td> " + adapterItem.AssociationMode + "&nbsp;<br /></td>";
        output += "<td><b>&nbsp;Association Type:&nbsp;</b></td><td> " + adapterItem.AssociationType + "&nbsp;<br /></td></tr>";
        //output += "<b>Transmit Rate :&nbsp;</b> " + getDataRateInMbps(adapterItem.TransmitRate)/100 + " (Mbps)&nbsp;<br />";
        //output += "<b>Receive Rate:&nbsp;</b> " + getDataRateInMbps(adapterItem.ReceiveRate)/100 + " (Mbps)&nbsp;<br />";
        output += "<tr><td><b>Driver Version:&nbsp;</b></td><td> " + adapterItem.DriverVersion + "&nbsp;<br /></td>";

        /*
        var transmitPower = ""+adapterItem.TransmitPower+"&nbsp; mW";
        if(-1==adapterItem.TransmitPower
        || 0 == adapterItem.TransmitPower)
        {
        transmitPower = "NA";
        }
        */

        output += "<td><b>&nbsp;MAC Address:&nbsp;</b></td><td title='" + adapterItem.MacAddress + "'> " + adapterItem.DecodedMacAddress + "<br /></td></tr>";
        var chiperAlgorithms = adapterItem.SecutityAlgorithms.split(',');
        var authAlgo = chiperAlgorithms[0];
        var chiperAlgo = chiperAlgorithms[1];

        output += "<tr><td><b>Authentication:&nbsp;</b></td><td> " + authAlgo + "&nbsp;<br /></td>";
        output += "<td><b>&nbsp;Encryption:&nbsp;</b></td><td> " + chiperAlgo + "<br /></td></tr>";
        output += "<tr><td><b>Transmit Rate:&nbsp;</b></td><td> " + adapterItem.TransmitRate / 1000 + "&nbsp; Mbits/sec<br /></td>";
        output += "<td><b>&nbsp;Receive Rate:&nbsp;</b></td><td> " + adapterItem.ReceiveRate / 1000 + "&nbsp; Mbits/sec<br /></td></tr>";
        //output += "<tr><td></td><td></td><td></td><td><center> <input type='button' onclick='disconnectAdapter("+ i +");' value='Disconnect' title='Click to show connect/disconnect UI'></input>  </center></td></tr>";
        output += "</table>";

        var networks = adapterItem.GetNetworks();
        var ntworktablename = "AvaillableNetworksTable" + i;
        var availlableNetworksTable = makeAvaillableNetworksTable(ntworktablename, adapterItem, networks, i);
        var multicastTablename = "StatisticTableMulticast" + i;
        var statisticTableMulticast = makeStatisticTableMulticast(multicastTablename, adapterItem);
        var unicastTablename = "StatisticTableUnicast" + i;
        var statisticTableUnicast = makeStatisticTableUnicast(unicastTablename, adapterItem);
        var physycalTablePrefix = "phisicalTable" + i + "_";
        var statisticTablePhysical = makeStatisticTablePhysycal(physycalTablePrefix, adapterItem);

        var statTabID = "adapterStatisticTab" + i;
        var arrButtonsStat = new Array("Unicast Counters", "Multicast Counters", "Physical Counters");
        var arrStatContent = new Array(
                    statisticTableUnicast,
                    statisticTableMulticast,
                    statisticTablePhysical
            );
        var selectedIdx = 0;

        if (tabBuilder.HaveControl(statTabID)) {
            selectedIdx = tabBuilder.GetSelected(statTabID);
        }
        tabBuilder.AddControl(statTabID, selectedIdx, arrButtonsStat, arrStatContent);
        var statisticTable = tabBuilder.GetTabHtml(statTabID, false);
        var terminology = makeTerminologyTable("termiology" + i);
        var adapterDataTabID = "adapterDataTab" + i;
        var arrButtons = new Array("Networks", "Statistics", "Glossary");
        var arrContent = new Array(
                availlableNetworksTable,
                statisticTable,
                terminology
            );
        selectedIdx = 0;
        if (tabBuilder.HaveControl(adapterDataTabID)) {
            selectedIdx = tabBuilder.GetSelected(adapterDataTabID);
        }
        tabBuilder.AddControl(adapterDataTabID, selectedIdx, arrButtons, arrContent);

        var comments = "";
        //System.Debug.outputString("selectedIdx:" + selectedIdx + ";i:" + i + ";openedAdapterIndex:" + openedAdapterIndex + ";");
        if (0 == selectedIdx && i == openedAdapterIndex) {
            comments = "<div style='font-size: x-small; font-weight:bold; display:inline;'>&nbsp;&nbsp;Click on SSID to enter Locate mode</div>";
        }

        var tabOutput = tabBuilder.GetTabHtml(adapterDataTabID, false, comments);
        //System.Debug.outputString("tabOutput "+i+ "-> "+tabOutput);

        output += tabOutput;
        if (count == 1) {
            output += "</div></div>";
        }
        return output;

    }

    function BuildFlyout() {
        var output = "";
        // Get the number of unread items
        var count = GetWrapi().GetNumWireless;
        tabBuilder.SetSkinColors(arrSkinColors);

        var closeButtonText = "Close";
        var ai = System.Gadget.Settings.read("adapterIndex");
        var ad = GetWrapi().GetAdapter(ai);
        if (ad.IsLocateMode()) {
            closeButtonText = "Close";
        }
        //style='background-color=red;'
        output += "<div id='ControlButtons'> <table width='99%'><tr>";

        //if(!ad.IsLocateMode()){
        output += "<td align = 'right' width='55%'><input type='button' onclick='closeflyout();' value='" + closeButtonText + "' title='Click to close fly-out'/>";

        /*
        }else{
        output +="<td align = 'right' width='55%'>";
        }
        */
        output += "<input type='button' onclick='refreshflyout();' value='Refresh' title='Click to refresh content'/></td>" +

        "<td align = 'right'><input type='button' onclick='showconnectui();' value='Connect/Disconnect' title='Click to show connect/disconnect UI'></input>" +
        "</td></tr></table></div></br>";

        if (count > 1) {
            adapterIndex = System.Gadget.Settings.read("adapterIndex");
            adapterIndex += 1;
            output += "<b>Number of adapters:</b> " + count + "&nbsp;<b>Selected adapter:</b> " + adapterIndex + "</br>";
        }


        //System.Debug.outputString("FlyoutBuilder has "+ count+" adapters")
        //System.Debug.outputString("openedAdapterIndex="+openedAdapterIndex);
        var adapterContents = new Array();
        var aidx = 0;
        for (var i = 0; i < count; i++) {
            var adapterItem = GetWrapi().GetAdapter(i);

            var adapterOutput = BuildSingleAdapterOutput(i, adapterItem);
            if (null == adapterOutput) continue;

            adapterContents[aidx] = adapterOutput;
            //System.Debug.outputString("------------------> Adapter "+aidx+" <----------------------");
            //System.Debug.outputString(adapterOutput);
            //System.Debug.outputString("<------------------ Adapter "+aidx+" ---------------------->");
            aidx = aidx + 1;
        }

        if (1 == aidx) {
            return output + adapterContents[0];
        }
        var arrButtons = new Array();
        for (var b = 0; b < aidx; ++b) {
            arrButtons[b] = "Adapter " + (b + 1);
        }
        var selectedIdx = adapterIndex - 1;
        var adaptersTabID = "adaptersTab";
        if (tabBuilder.HaveControl(adaptersTabID)) {

            selectedIdx = tabBuilder.GetSelected(adaptersTabID);

        }
        //System.Debug.outputString(" Selected ADAPTER IDX: " + selectedIdx);
        tabBuilder.AddControl(adaptersTabID, selectedIdx, arrButtons, adapterContents);

        var tabOutput = tabBuilder.GetTabHtml(adaptersTabID, false);
        return output + tabOutput;
    }

}
